/*
 * This file is part of smolbote. It's copyrighted by the contributors recorded
 * in the version control history of the file, available from its original
 * location: https://neueland.iserlohn-fortress.net/gitea/aqua/smolbote
 *
 * SPDX-License-Identifier: GPL-3.0
 */

#include "xbel.h"
#include "bookmarkitem.h"
#include <QXmlStreamReader>
#include <QXmlStreamWriter>
#include <QDateTime>

inline void readChildElements(QXmlStreamReader &reader, BookmarkItem *parent)
{
    while(reader.readNextStartElement()) {
        if(reader.name() == "title") {
            parent->setData(BookmarkItem::Title, reader.readElementText());

        } else if(reader.name() == "dateAdded") {
            parent->setData(BookmarkItem::DateAdded, QDateTime::fromString(reader.readElementText(), Qt::RFC2822Date));

        } else if(reader.name() == "lastModified") {
            parent->setData(BookmarkItem::LastModified, QDateTime::fromString(reader.readElementText(), Qt::RFC2822Date));

        } else if(reader.name() == "tags") {
            parent->setData(BookmarkItem::Tags, reader.readElementText().split(";"));

        } else if(reader.name() == "description") {
            parent->setData(BookmarkItem::Description, reader.readElementText());

        } else if(reader.name() == "folder") {
            auto *item = new BookmarkItem({}, BookmarkItem::Folder, parent);
            item->setExpanded(!(reader.attributes().value("folded") == QLatin1String("yes")));
            parent->appendChild(item);
            readChildElements(reader, item);

        } else if(reader.name() == "bookmark") {
            auto *item = new BookmarkItem({}, BookmarkItem::Bookmark, parent);
            item->setData(BookmarkItem::Href, reader.attributes().value("href").toString());
            parent->appendChild(item);
            readChildElements(reader, item);

        } else {
            reader.skipCurrentElement();
        }
    }
}

void Xbel::read(QIODevice *device, BookmarkItem *item)
{
    QXmlStreamReader qXmlStreamReader(device);

    if(qXmlStreamReader.readNextStartElement()) {
        if(!(qXmlStreamReader.name() == "xbel" && qXmlStreamReader.attributes().value("version") == "1.0")) {
            return;
        }

        readChildElements(qXmlStreamReader, item);
    }
}

inline void writeCommon(QXmlStreamWriter &writer, const BookmarkItem *item)
{
    writer.writeTextElement("title", item->data(BookmarkItem::Title).toString());

    const auto dateAdded = item->data(BookmarkItem::DateAdded);
    if(!dateAdded.isNull())
        writer.writeTextElement("dateAdded", dateAdded.toDateTime().toString(Qt::RFC2822Date));

    const auto lastModified = item->data(BookmarkItem::LastModified);
    if(!lastModified.isNull())
        writer.writeTextElement("lastModified", lastModified.toDateTime().toString(Qt::RFC2822Date));
}

inline void writeChildElements(QXmlStreamWriter &writer, const BookmarkItem *item)
{
    switch(item->type()) {
    case BookmarkItem::Root:
        for(int i = 0; i < item->childCount(); ++i) {
            writeChildElements(writer, item->child(i));
        }
        break;

    case BookmarkItem::Folder:
        writer.writeStartElement("folder");
        writer.writeAttribute("folded", !item->isExpanded() ? "yes" : "no");
        writeCommon(writer, item);
        if(!item->data(BookmarkItem::Tags).isNull())
            writer.writeTextElement("tags", item->data(BookmarkItem::Tags).toStringList().join(";"));
        if(!item->data(BookmarkItem::Description).isNull())
            writer.writeTextElement("description", item->data(BookmarkItem::Description).toString());

        for(int i = 0; i < item->childCount(); ++i) {
            writeChildElements(writer, item->child(i));
        }

        writer.writeEndElement();
        break;

    case BookmarkItem::Bookmark:
        writer.writeStartElement("bookmark");
        writer.writeAttribute("href", item->data(BookmarkItem::Href).toString());
        writeCommon(writer, item);
        if(!item->data(BookmarkItem::Tags).isNull())
            writer.writeTextElement("tags", item->data(BookmarkItem::Tags).toStringList().join(";"));
        if(!item->data(BookmarkItem::Description).isNull())
            writer.writeTextElement("description", item->data(BookmarkItem::Description).toString());

        writer.writeEndElement();
        break;
    }
}

void Xbel::write(QIODevice *device, const BookmarkItem *item)
{
    QXmlStreamWriter xmlWriter(device);
    xmlWriter.setAutoFormatting(true);

    xmlWriter.writeStartDocument();
    xmlWriter.writeDTD("<!DOCTYPE xbel>");

    xmlWriter.writeStartElement("xbel");
    xmlWriter.writeAttribute("version", "1.0");

    writeChildElements(xmlWriter, item);

    xmlWriter.writeEndDocument();
}
