#include "urlfilter.h"
#include "adblock/adblocklist.h"
#include <gtest/gtest.h>
#include <QFile>

AdBlockList *list = nullptr;

TEST(AdBlockList, MetaData) {
    EXPECT_STREQ(qUtf8Printable(list->metadata("Homepage")), "http://example.com/");
    EXPECT_STREQ(qUtf8Printable(list->metadata("Title")), "FooList");
    EXPECT_STREQ(qUtf8Printable(list->metadata("Expires")), "5 days");
    EXPECT_STREQ(qUtf8Printable(list->metadata("Redirect")), "http://example.com/list.txt");
    EXPECT_STREQ(qUtf8Printable(list->metadata("Version")), "1234");
}

TEST(AdBlockList, BasicFilter) {
    // Rule: /banner/*/img^
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/foo/img"),            QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/foo/bar/img?param"),  QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner//img/foo"),           QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::Block);

    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/foo.png"),        QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::NotMatched);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/img"),            QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::NotMatched);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/foo/imgraph"),    QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::NotMatched);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/foo/img.gif"),    QWebEngineUrlRequestInfo::ResourceTypeImage).first, UrlFilter::NotMatched);

    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner/ads/img.png"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

TEST(AdBlockList, MatchBeginningEnd) {
    // Rule: |http://beginning-pattern.com
    EXPECT_EQ(list->match(QUrl(), QUrl("http://beginning-pattern.com"),  QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("https://beginning-pattern.com"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
    // Rule: end-pattern|
    EXPECT_EQ(list->match(QUrl(), QUrl("https://endpattern.com/end-pattern"),     QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("https://endpattern.com/end-pattern/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

TEST(AdBlockList, Domain) {
    // Rule: ||ads.example.com^
    EXPECT_EQ(list->match(QUrl(), QUrl("http://ads.example.com/foo.gif"),           QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://server1.ads.example.com/foo.gif"),   QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("https://ads.example.com:8000/"),            QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);

    EXPECT_EQ(list->match(QUrl(), QUrl("http://ads.example.com.ua/foo.gif"),                    QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/redirect/http://ads.example.com/"),  QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

TEST(AdBlockList, RegularExpression) {
    // Rule: /banner\d+/
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner123"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banner321"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://example.com/banners"),   QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

TEST(AdBlockList, MatchCase) {
    // Rule: matchThisCase$match-case
    EXPECT_EQ(list->match(QUrl(), QUrl("http://matchcase.com/matchThisCase"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl(), QUrl("http://matchcase.com/MatchThisCase"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

TEST(AdBlockList, DomainOption) {
    // Rule: domain-limited-string$domain=example.com
    EXPECT_EQ(list->match(QUrl("https://example.com"), QUrl("https://example.com/domain-limited-string/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl("https://example.com"), QUrl("https://example.com/another-domain-string/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
    EXPECT_EQ(list->match(QUrl("https://another.com"), QUrl("https://example.com/domain-limited-string/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);

    //Rule: exception-limited-string$domain=~example.com
    EXPECT_EQ(list->match(QUrl("https://another.com"), QUrl("https://example.com/exception-limited-string/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::Block);
    EXPECT_EQ(list->match(QUrl("https://example.com"), QUrl("https://example.com/exception-limited-string/foo"), QWebEngineUrlRequestInfo::ResourceTypeMainFrame).first, UrlFilter::NotMatched);
}

int main(int argc, char **argv) {
    QFile f("adblock.txt");
    if(!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug("Could not open list");
        return -1;
    }

    list = new AdBlockList(&f);
    f.close();

    qDebug("Parsed %i rules", list->ruleCount());

    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}

