/*
 * This file is part of smolbote. It's copyrighted by the contributors recorded
 * in the version control history of the file, available from its original
 * location: https://neueland.iserlohn-fortress.net/cgit/smolbote
 *
 * SPDX-License-Identifier: GPL-3.0
 */

#include "profileview.h"
#include "ui_profileview.h"
#include <QDateTime>
#include <QWebEngineCookieStore>
#include <QWebEngineSettings>

inline void connectSetting(QCheckBox *checkBox, QWebEngineProfile *profile, QWebEngineSettings::WebAttribute attr)
{
    checkBox->setChecked(profile->settings()->testAttribute(attr));
    QObject::connect(checkBox, &QCheckBox::clicked, [profile, attr](bool checked) {
        profile->settings()->setAttribute(attr, checked);
    });
}

ProfileView::ProfileView(const QString &name, QWebEngineProfile *profile, QSettings *settings, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::ProfileView)
{
    Q_CHECK_PTR(profile);
    m_profile = profile;
    ui->setupUi(this);

    // general tab
    ui->name->setText(name);
    ui->offTheRecord->setChecked(profile->isOffTheRecord());

    ui->settings_table->connect(settings);

    // http tab
    ui->userAgent->setPlainText(m_profile->httpUserAgent());
    connect(ui->userAgent, &QPlainTextEdit::textChanged, profile, [this, profile]() {
        profile->setHttpUserAgent(ui->userAgent->toPlainText());
    });

    ui->acceptLanguage->setPlainText(m_profile->httpAcceptLanguage());
    connect(ui->acceptLanguage, &QPlainTextEdit::textChanged, profile, [this, profile]() {
        profile->setHttpAcceptLanguage(ui->acceptLanguage->toPlainText());
    });

    ui->headers_table->connect(settings, "headers");

    ui->cacheType->setCurrentIndex(m_profile->httpCacheType());
    connect(ui->cacheType, QOverload<int>::of(&QComboBox::currentIndexChanged), profile, [profile](int index) {
        profile->setHttpCacheType(static_cast<QWebEngineProfile::HttpCacheType>(index));
    });

    ui->cacheSize->setText(QString::number(m_profile->httpCacheMaximumSize()));
    connect(ui->cacheSize, &QLineEdit::textChanged, profile, [profile](const QString &text) {
        profile->setHttpCacheMaximumSize(text.toInt());
    });

    ui->cookiePolicy->setCurrentIndex(m_profile->persistentCookiesPolicy());
    connect(ui->cookiePolicy, QOverload<int>::of(&QComboBox::currentIndexChanged), profile, [profile](int index) {
        profile->setPersistentCookiesPolicy(static_cast<QWebEngineProfile::PersistentCookiesPolicy>(index));
    });

    connect(ui->clearCache_pushButton, &QPushButton::clicked, profile, &QWebEngineProfile::clearHttpCache);
    connect(ui->clearHistory_pushButton, &QPushButton::clicked, profile, &QWebEngineProfile::clearAllVisitedLinks);

    ui->storagePath_lineEdit->setText(m_profile->persistentStoragePath());
    ui->cachePath_lineEdit->setText(m_profile->cachePath());

    if(profile->isOffTheRecord()) {
        ui->cacheType->setEnabled(false);
        ui->cacheSize->setEnabled(false);
        ui->storagePath_lineEdit->setEnabled(false);
        ui->cachePath_lineEdit->setEnabled(false);
        ui->cookiePolicy->setEnabled(false);
    }
    // settings tab
    connectSetting(ui->autoloadImages, m_profile, QWebEngineSettings::AutoLoadImages);
    connectSetting(ui->autoloadIcons, m_profile, QWebEngineSettings::AutoLoadIconsForPage);

    connectSetting(ui->javascriptEnabled, m_profile, QWebEngineSettings::JavascriptEnabled);
    connectSetting(ui->javascriptCanAccessClipboard, m_profile, QWebEngineSettings::JavascriptCanAccessClipboard);
    connectSetting(ui->javascriptCanOpenWindows, m_profile, QWebEngineSettings::JavascriptCanOpenWindows);
    connectSetting(ui->javascriptCanActivateWindows, m_profile, QWebEngineSettings::AllowWindowActivationFromJavaScript);

    connectSetting(ui->linksIncludedInFocusChain, m_profile, QWebEngineSettings::LinksIncludedInFocusChain);
    connectSetting(ui->spatialNavigationEnabled, m_profile, QWebEngineSettings::SpatialNavigationEnabled);
    connectSetting(ui->focusOnNavigationEnabled, m_profile, QWebEngineSettings::FocusOnNavigationEnabled);
    connectSetting(ui->touchIconsEnabled, m_profile, QWebEngineSettings::TouchIconsEnabled);

    connectSetting(ui->showScrollBars, m_profile, QWebEngineSettings::ShowScrollBars);
    connectSetting(ui->scrollAnimatorEnabled, m_profile, QWebEngineSettings::ScrollAnimatorEnabled);
    connectSetting(ui->errorPagesEnabled, m_profile, QWebEngineSettings::ErrorPageEnabled);

    connectSetting(ui->localContentCanAccessRemoteUrls, m_profile, QWebEngineSettings::LocalContentCanAccessRemoteUrls);
    connectSetting(ui->localContentCanAccessFileUrls, m_profile, QWebEngineSettings::LocalContentCanAccessFileUrls);

    connectSetting(ui->localStorageEnabled, m_profile, QWebEngineSettings::LocalStorageEnabled);
    connectSetting(ui->webglEnabled, m_profile, QWebEngineSettings::WebGLEnabled);
    connectSetting(ui->canvasEnabled, m_profile, QWebEngineSettings::Accelerated2dCanvasEnabled);

    connectSetting(ui->xssAuditingEnabled, m_profile, QWebEngineSettings::XSSAuditingEnabled);
    connectSetting(ui->hyperlinkAuditingEnabled, m_profile, QWebEngineSettings::HyperlinkAuditingEnabled);

    connectSetting(ui->allowRunningInsecureContent, m_profile, QWebEngineSettings::AllowRunningInsecureContent);
    connectSetting(ui->allowGeolocationOnInsecureContent, m_profile, QWebEngineSettings::AllowGeolocationOnInsecureOrigins);
    connectSetting(ui->ppapiPluginsEnabled, m_profile, QWebEngineSettings::PluginsEnabled);
    connectSetting(ui->fullscreenSupportEnabled, m_profile, QWebEngineSettings::FullScreenSupportEnabled);
    connectSetting(ui->screenCaptureEnabled, m_profile, QWebEngineSettings::ScreenCaptureEnabled);
    connectSetting(ui->printElementBackgrounds, m_profile, QWebEngineSettings::PrintElementBackgrounds);

    // cookies tab
    for(auto &raw_cookies : profile->property("cookies").toList()) {
        for(auto &cookie : QNetworkCookie::parseCookies(raw_cookies.toByteArray())) {
            cookieAdded(cookie);
        }
    }

    connect(profile->cookieStore(), &QWebEngineCookieStore::cookieAdded, this, &ProfileView::cookieAdded);
    connect(profile->cookieStore(), &QWebEngineCookieStore::cookieRemoved, this, &ProfileView::cookieRemoved);

    connect(ui->cookies_deleteSession, &QPushButton::clicked, profile->cookieStore(), &QWebEngineCookieStore::deleteSessionCookies);
    connect(ui->cookies_deleteAll, &QPushButton::clicked, profile->cookieStore(), &QWebEngineCookieStore::deleteAllCookies);

    connect(ui->cookies_delete, &QPushButton::clicked, [this, profile]() {
        for(auto &list : ui->cookies->selectedRanges()) {
            for(int i = list.bottomRow(); i >= list.topRow(); --i) {
                auto cookie = ui->cookies->item(i, 0)->data(Qt::UserRole).value<QNetworkCookie>();
                profile->cookieStore()->deleteCookie(cookie);
            }
        }
    });

    connect(ui->cookies_reload, &QPushButton::clicked, [this, profile]() {
        ui->cookies->clearContents();
        ui->cookies->setRowCount(0);
        qDebug("loadAllCookies called!");
        profile->cookieStore()->loadAllCookies();
    });

}

ProfileView::~ProfileView()
{
    delete ui;
}

void ProfileView::cookieAdded(const QNetworkCookie &cookie)
{
    auto index = ui->cookies->rowCount();
    ui->cookies->setRowCount(index + 1);

    auto *item = new QTableWidgetItem(QString(cookie.name()));
    item->setData(Qt::UserRole, QVariant::fromValue(cookie));
    ui->cookies->setItem(index, 0, item);
    ui->cookies->setItem(index, 1, new QTableWidgetItem(cookie.domain()));
    ui->cookies->setItem(index, 2, new QTableWidgetItem(cookie.path()));
    if(cookie.isSessionCookie()) {
        ui->cookies->setItem(index, 3, new QTableWidgetItem(tr("session")));
    } else {
        ui->cookies->setItem(index, 3, new QTableWidgetItem(cookie.expirationDate().toString(Qt::RFC2822Date)));
    }
}

void ProfileView::cookieRemoved(const QNetworkCookie &cookie)
{
    qDebug("cookieRemoved called");

    for(int i = 0; i < ui->cookies->rowCount(); ++i) {
        auto *item = ui->cookies->item(i, 0);
        if(item->data(Qt::UserRole).value<QNetworkCookie>().hasSameIdentifier(cookie)) {
            qDebug("removing cookie on row %i", i);
            ui->cookies->removeRow(i);
            return;
        }
        qDebug("cookie on row %i doesn't have the same identifier", i);
    }
}
